repo system 0 testtags <inline>
#>=Pkg: A 1 1 i586
#>=Pkg: A 1 1 x86_64
repo test 0 testtags <inline>
#>=Pkg: A 2 1 i586
#>=Pkg: A 2 1 x86_64
#>=Pkg: A 3 1 i586
#>=Pkg: A 4 1 x86_64
system x86_64 rpm system
poolflags implicitobsoleteusescolors
job update all packages
result transaction,problems <inline>
#>upgrade A-1-1.i586@system A-2-1.i586@test
#>upgrade A-1-1.x86_64@system A-2-1.x86_64@test
