/*
 * Copyright (c) 2014, SUSE Inc.
 *
 * This program is licensed under the BSD license, read LICENSE.BSD
 * for further information
 */

#ifndef POOL_PARSERPMRICHDEP_H
#define POOL_PARSERPMRICHDEP_H

#include "pool.h"

extern Id pool_parserpmrichdep(Pool *pool, const char *dep);

#endif
